@@ -2,15 +2,17 @@
 
 import math
 import os
+import re
 import sys
 import warnings
+from collections import OrderedDict
 
 import ruamel.yaml
 
 from osaca.semantics import MachineModel
 
 
-def sanity_check(arch: str, verbose=False):
+def sanity_check(arch: str, verbose=False, internet_check=False, output_file=sys.stdout):
     """
     Checks the database for missing TP/LT values, instructions might missing int the ISA DB and
     duplicate instructions.
@@ -19,6 +21,11 @@ def sanity_check(arch: str, verbose=False):
     :type arch: str
     :param verbose: verbose output flag, defaults to `False`
     :type verbose: bool, optional
+    :param internet_check: indicates if OSACA should try to look up the src/dst distribution in the internet, defaults to False
+    :type internet_check: boolean, optional
+    :param output_file: output stream specifying where to write output, defaults to :class:`sys.stdout`
+    :type output_file: stream, optional
+
     """
     # load arch machine model
     arch_mm = MachineModel(arch=arch)
@@ -35,11 +42,11 @@ def sanity_check(arch: str, verbose=False):
         missing_port_pressure,
         suspicious_instructions,
         duplicate_instr_arch,
-    ) = _check_sanity_arch_db(arch_mm, isa_mm)
+    ) = _check_sanity_arch_db(arch_mm, isa_mm, internet_check=internet_check)
     # check ISA DB entries
     duplicate_instr_isa, only_in_isa = _check_sanity_isa_db(arch_mm, isa_mm)
 
-    _print_sanity_report(
+    report = _get_sanity_report(
         num_of_instr,
         missing_throughput,
         missing_latency,
@@ -49,10 +56,12 @@ def sanity_check(arch: str, verbose=False):
         duplicate_instr_isa,
         only_in_isa,
         verbose=verbose,
+        colors=True if output_file == sys.stdout else False,
     )
+    print(report, file=output_file)
 
 
-def import_benchmark_output(arch, bench_type, filepath):
+def import_benchmark_output(arch, bench_type, filepath, output=sys.stdout):
     """
     Import benchmark results from micro-benchmarks.
 
@@ -62,6 +71,8 @@ def import_benchmark_output(arch, bench_type, filepath):
     :type bench_type: str
     :param filepath: filepath to the output file
     :type filepath: str
+    :param output: output stream to dump, defaults to sys.stdout
+    :type output: stream
     """
     supported_bench_outputs = ['ibench', 'asmbench']
     assert os.path.exists(filepath)
@@ -78,7 +89,10 @@ def import_benchmark_output(arch, bench_type, filepath):
     # write entries to DB
     for entry in db_entries:
         mm.set_instruction_entry(db_entries[entry])
-    sys.stdout.write(mm.dump())
+    if output is None:
+        print(mm.dump())
+    else:
+        mm.dump(stream=output)
 
 
 ##################
@@ -177,6 +191,10 @@ def _get_ibench_output(input_data, isa):
 
 
 def _validate_measurement(measurement, mode):
+    """
+    Check if latency has a maximum deviation of 0.05% and throughput is a reciprocal of a
+    an integer number.
+    """
     if mode == 'lt':
         if (
             math.floor(measurement) * 1.05 >= measurement
@@ -197,6 +215,7 @@ def _validate_measurement(measurement, mode):
 
 
 def _create_db_operand(operand, isa):
+    """Get DB operand by input string and ISA."""
     if isa == 'aarch64':
         return _create_db_operand_aarch64(operand)
     elif isa == 'x86':
@@ -204,16 +223,21 @@ def _create_db_operand(operand, isa):
 
 
 def _create_db_operand_aarch64(operand):
+    """Get DB operand for AArch64 by operand string."""
     if operand == 'i':
         return {'class': 'immediate', 'imd': 'int'}
     elif operand in 'wxbhsdq':
         return {'class': 'register', 'prefix': operand}
     elif operand.startswith('v'):
-        return {'class': 'register', 'prefix': 'v', 'shape': operand[1:2]}
+        return {
+            'class': 'register',
+            'prefix': 'v',
+            'shape': operand[1:2] if operand[1:2] != '' else 'd',
+        }
     elif operand.startswith('m'):
         return {
             'class': 'memory',
-            'base': 'gpr' if 'b' in operand else None,
+            'base': 'x' if 'b' in operand else None,
             'offset': 'imd' if 'o' in operand else None,
             'index': 'gpr' if 'i' in operand else None,
             'scale': 8 if 's' in operand else 1,
@@ -225,6 +249,7 @@ def _create_db_operand_aarch64(operand):
 
 
 def _create_db_operand_x86(operand):
+    """Get DB operand for AArch64 by operand string."""
     if operand == 'r':
         return {'class': 'register', 'name': 'gpr'}
     elif operand in 'xyz':
@@ -248,9 +273,105 @@ def _create_db_operand_x86(operand):
 ########################
 
 
-def _check_sanity_arch_db(arch_mm, isa_mm):
+def _scrape_from_felixcloutier(mnemonic):
+    """Scrape src/dst information from felixcloutier website and return information for user."""
+    import requests
+
+    try:
+        from bs4 import BeautifulSoup
+    except ImportError:
+        print(
+            'Module BeautifulSoup not installed. Fetching instruction form information '
+            'online requires BeautifulSoup.\nUse \'pip install bs4\' for installation.',
+            file=sys.stderr,
+        )
+        sys.exit(1)
+
+    index = 'https://www.felixcloutier.com/x86/index.html'
+    base_url = 'https://www.felixcloutier.com/x86/'
+    url = base_url + mnemonic.lower()
+
+    suspicious = True
+    operands = []
+
+    # GET website
+    r = requests.get(url=url)
+    if r.status_code == 200:
+        # Found result
+        operand_enc = BeautifulSoup(r.text, 'html.parser').find(
+            'h2', attrs={'id': 'instruction-operand-encoding'}
+        )
+        if operand_enc:
+            # operand encoding found, otherwise, no need to mark as suspicous
+            table = operand_enc.findNextSibling()
+            operands = _get_src_dst_from_table(table)
+    elif r.status_code == 404:
+        # Check for alternative href
+        index = BeautifulSoup(requests.get(url=index).text, 'html.parser')
+        alternatives = [ref for ref in index.findAll('a') if ref.text == mnemonic.upper()]
+        if len(alternatives) > 0:
+            # alternative(s) found, take first one
+            url = base_url + alternatives[0].attrs['href'][2:]
+            operand_enc = BeautifulSoup(requests.get(url=url).text, 'html.parser').find(
+                'h2', attrs={'id': 'instruction-operand-encoding'}
+            )
+            if operand_enc:
+                # operand encoding found, otherwise, no need to mark as suspicous
+                table = (
+                    operand_enc.findNextSibling()
+                )
+                operands = _get_src_dst_from_table(table)
+    if operands:
+        # Found src/dst assignment for NUM_OPERANDS
+        if not any(['r' in x and 'w' in x for x in operands]):
+            suspicious = False
+    return (suspicious, ' '.join(operands))
+
+
+def _get_src_dst_from_table(table, num_operands=2):
+    """Prettify bs4 table object to string for user"""
+    # Parse table
+    header = [''.join(x.string.lower().split()) for x in table.find('tr').findAll('td')]
+    data = table.findAll('tr')[1:]
+    data_dict = OrderedDict()
+    for i, row in enumerate(data):
+        data_dict[i] = {}
+        for j, col in enumerate(row.findAll('td')):
+            if col.string != 'NA':
+                data_dict[i][header[j]] = col.string
+    # Get only the instruction forms with 2 operands
+    num_ops = [_get_number_of_operands(row) for _, row in data_dict.items()]
+    if num_operands in num_ops:
+        row = data_dict[num_ops.index(num_operands)]
+        reads_writes = []
+        for i in range(1, num_operands + 1):
+            m = re.search(r'(\([^\(\)]+\))', row['operand{}'.format(i)])
+            if not m:
+                # no parentheses (probably immediate operand), assume READ
+                reads_writes.append('(r)')
+                continue
+            reads_writes.append(''.join(m.group(0).split()))
+        # reverse reads_writes for AT&T syntax
+        reads_writes.reverse()
+        return reads_writes
+    return []
+
+
+def _get_number_of_operands(data_dict_row):
+    """Return the number of `Operand [X]` attributes in row"""
+    num = 0
+    for i in range(1, 5):
+        if 'operand{}'.format(i) in [''.join(x.split()).lower() for x in data_dict_row]:
+            num += 1
+    return num
+
+
+def _check_sanity_arch_db(arch_mm, isa_mm, internet_check=True):
+    """Do sanity check for ArchDB by given ISA."""
+    # prefixes of instruction forms which we assume to have non-default operands
     suspicious_prefixes_x86 = ['vfm', 'fm']
     suspicious_prefixes_arm = ['fml', 'ldp', 'stp', 'str']
+    # already known to be default-operand instruction forms with 2 operands
     if arch_mm.get_ISA().lower() == 'aarch64':
         suspicious_prefixes = suspicious_prefixes_arm
     if arch_mm.get_ISA().lower() == 'x86':
@@ -262,6 +383,7 @@ def _check_sanity_arch_db(arch_mm, isa_mm):
     missing_port_pressure = []
     suspicious_instructions = []
     duplicate_instr_arch = []
+    duplicate_strings = []
 
     for instr_form in arch_mm['instruction_forms']:
         # check value in DB entry
@@ -278,14 +400,32 @@ def _check_sanity_arch_db(arch_mm, isa_mm):
                 if isa_mm.get_instruction(instr_form['name'], instr_form['operands']) is None:
                     # if not, mark them as suspicious and print it on the screen
                     suspicious_instructions.append(instr_form)
+        # instr forms with less than 3 operands might need an ISA DB entry due to src_reg operands
+        if (
+            len(instr_form['operands']) < 3
+            and len(instr_form['operands']) > 1
+            and 'mov' not in instr_form['name'].lower()
+            and not instr_form['name'].lower().startswith('j')
+            and instr_form not in suspicious_instructions
+            and isa_mm.get_instruction(instr_form['name'], instr_form['operands']) is None
+        ):
+            # validate with data from internet if connected flag is set
+            if internet_check:
+                is_susp, info_string = _scrape_from_felixcloutier(instr_form['name'])
+                if is_susp:
+                    instr_form['note'] = info_string
+                    suspicious_instructions.append(instr_form)
+            else:
+                suspicious_instructions.append(instr_form)
         # check for duplicates in DB
         if arch_mm._check_for_duplicate(instr_form['name'], instr_form['operands']):
             duplicate_instr_arch.append(instr_form)
     # every entry exists twice --> uniquify
     tmp_list = []
-    for i in range(0, len(duplicate_instr_arch)):
+    for _ in range(0, len(duplicate_instr_arch)):
         tmp = duplicate_instr_arch.pop()
-        if tmp not in duplicate_instr_arch:
+        if _get_full_instruction_name(tmp).lower() not in duplicate_strings:
+            duplicate_strings.append(_get_full_instruction_name(tmp).lower())
             tmp_list.append(tmp)
     duplicate_instr_arch = tmp_list
     return (
@@ -298,6 +438,7 @@ def _check_sanity_arch_db(arch_mm, isa_mm):
 
 
 def _check_sanity_isa_db(arch_mm, isa_mm):
+    """Do sanity check for an ISA DB."""
     # returned lists
     duplicate_instr_isa = []
     only_in_isa = []
@@ -320,88 +461,83 @@ def _check_sanity_isa_db(arch_mm, isa_mm):
     return duplicate_instr_isa, only_in_isa
 
 
-def _print_sanity_report(
-    total, m_tp, m_l, m_pp, suspic_instr, dup_arch, dup_isa, only_isa, verbose=False
+def _get_sanity_report(
+    total, m_tp, m_l, m_pp, suspic_instr, dup_arch, dup_isa, only_isa, verbose=False, colors=False
 ):
+    """Get sanity summary report."""
+    s = ''
     # non-verbose summary
-    print('SUMMARY\n----------------------')
-    print(
-        '{}% ({}/{}) of instruction forms have no throughput value.'.format(
-            round(100 * len(m_tp) / total), len(m_tp), total
-        )
+    s += 'SUMMARY\n----------------------\n'
+    s += '{}% ({}/{}) of instruction forms have no throughput value.\n'.format(
+        round(100 * len(m_tp) / total), len(m_tp), total
     )
-    print(
-        '{}% ({}/{}) of instruction forms have no latency value.'.format(
-            round(100 * len(m_l) / total), len(m_l), total
-        )
+    s += '{}% ({}/{}) of instruction forms have no latency value.\n'.format(
+        round(100 * len(m_l) / total), len(m_l), total
     )
-    print(
-        '{}% ({}/{}) of instruction forms have no port pressure assignment.'.format(
-            round(100 * len(m_pp) / total), len(m_pp), total
-        )
+    s += '{}% ({}/{}) of instruction forms have no port pressure assignment.\n'.format(
+        round(100 * len(m_pp) / total), len(m_pp), total
     )
-    print(
-        '{}% ({}/{}) of instruction forms might miss an ISA DB entry.'.format(
-            round(100 * len(suspic_instr) / total), len(suspic_instr), total
-        )
+    s += '{}% ({}/{}) of instruction forms might miss an ISA DB entry.\n'.format(
+        round(100 * len(suspic_instr) / total), len(suspic_instr), total
     )
-    print('{} duplicate instruction forms in uarch DB.'.format(len(dup_arch)))
-    print('{} duplicate instruction forms in ISA DB.'.format(len(dup_isa)))
-    print(
+    s += '{} duplicate instruction forms in uarch DB.\n'.format(len(dup_arch))
+    s += '{} duplicate instruction forms in ISA DB.\n'.format(len(dup_isa))
+    s += (
         '{} instruction forms in ISA DB are not referenced by instruction '.format(len(only_isa))
-        + 'forms in uarch DB.'
+        + 'forms in uarch DB.\n'
     )
-    print('----------------------\n')
+    s += '----------------------\n'
     # verbose version
     if verbose:
-        _print_sanity_report_verbose(
-            total, m_tp, m_l, m_pp, suspic_instr, dup_arch, dup_isa, only_isa
+        s += _get_sanity_report_verbose(
+            total, m_tp, m_l, m_pp, suspic_instr, dup_arch, dup_isa, only_isa, colors=colors
         )
+    return s
 
 
-def _print_sanity_report_verbose(
-    total, m_tp, m_l, m_pp, suspic_instr, dup_arch, dup_isa, only_isa
+def _get_sanity_report_verbose(
+    total, m_tp, m_l, m_pp, suspic_instr, dup_arch, dup_isa, only_isa, colors=False
 ):
-    BRIGHT_CYAN = '\033[1;36;1m'
-    BRIGHT_BLUE = '\033[1;34;1m'
-    BRIGHT_RED = '\033[1;31;1m'
-    BRIGHT_MAGENTA = '\033[1;35;1m'
-    BRIGHT_YELLOW = '\033[1;33;1m'
-    CYAN = '\033[36m'
-    YELLOW = '\033[33m'
-    WHITE = '\033[0m'
+    """Get the verbose part of the sanity report with all missing instruction forms."""
+    BRIGHT_CYAN = '\033[1;36;1m' if colors else ''
+    BRIGHT_BLUE = '\033[1;34;1m' if colors else ''
+    BRIGHT_RED = '\033[1;31;1m' if colors else ''
+    BRIGHT_MAGENTA = '\033[1;35;1m' if colors else ''
+    BRIGHT_YELLOW = '\033[1;33;1m' if colors else ''
+    CYAN = '\033[36m' if colors else ''
+    YELLOW = '\033[33m' if colors else ''
+    WHITE = '\033[0m' if colors else ''
 
-    print('Instruction forms without throughput value:\n' if len(m_tp) != 0 else '', end='')
-    for instr_form in m_tp:
-        print('{}{}{}'.format(BRIGHT_BLUE, _get_full_instruction_name(instr_form), WHITE))
-    print('Instruction forms without latency value:\n' if len(m_l) != 0 else '', end='')
-    for instr_form in m_l:
-        print('{}{}{}'.format(BRIGHT_RED, _get_full_instruction_name(instr_form), WHITE))
-    print(
-        'Instruction forms without port pressure assignment:\n' if len(m_pp) != 0 else '', end=''
+    s = ''
+    s += 'Instruction forms without throughput value:\n' if len(m_tp) != 0 else ''
+    for instr_form in sorted(m_tp, key=lambda i: i['name']):
+        s += '{}{}{}\n'.format(BRIGHT_BLUE, _get_full_instruction_name(instr_form), WHITE)
+    s += 'Instruction forms without latency value:\n' if len(m_l) != 0 else ''
+    for instr_form in sorted(m_l, key=lambda i: i['name']):
+        s += '{}{}{}\n'.format(BRIGHT_RED, _get_full_instruction_name(instr_form), WHITE)
+    s += 'Instruction forms without port pressure assignment:\n' if len(m_pp) != 0 else ''
+    for instr_form in sorted(m_pp, key=lambda i: i['name']):
+        s += '{}{}{}\n'.format(BRIGHT_MAGENTA, _get_full_instruction_name(instr_form), WHITE)
+    s += 'Instruction forms which might miss an ISA DB entry:\n' if len(suspic_instr) != 0 else ''
+    for instr_form in sorted(suspic_instr, key=lambda i: i['name']):
+        s += '{}{}{}{}\n'.format(
+            BRIGHT_CYAN,
+            _get_full_instruction_name(instr_form),
+            ' -- ' + instr_form['note'] if 'note' in instr_form else '',
+            WHITE,
+        )
+    s += 'Duplicate instruction forms in uarch DB:\n' if len(dup_arch) != 0 else ''
+    for instr_form in sorted(dup_arch, key=lambda i: i['name']):
+        s += '{}{}{}\n'.format(YELLOW, _get_full_instruction_name(instr_form), WHITE)
+    s += 'Duplicate instruction forms in ISA DB:\n' if len(dup_isa) != 0 else ''
+    for instr_form in sorted(dup_isa, key=lambda i: i['name']):
+        s += '{}{}{}\n'.format(BRIGHT_YELLOW, _get_full_instruction_name(instr_form), WHITE)
+    s += (
+        'Instruction forms existing in ISA DB but not in uarch DB:\n' if len(only_isa) != 0 else ''
     )
-    for instr_form in m_pp:
-        print('{}{}{}'.format(BRIGHT_MAGENTA, _get_full_instruction_name(instr_form), WHITE))
-    print(
-        'Instruction forms which might miss an ISA DB entry:\n' if len(suspic_instr) != 0 else '',
-        end='',
-    )
-    for instr_form in suspic_instr:
-        print('{}{}{}'.format(BRIGHT_CYAN, _get_full_instruction_name(instr_form), WHITE))
-    print('Duplicate instruction forms in uarch DB:\n' if len(dup_arch) != 0 else '', end='')
-    for instr_form in dup_arch:
-        print('{}{}{}'.format(YELLOW, _get_full_instruction_name(instr_form), WHITE))
-    print('Duplicate instruction forms in ISA DB:\n' if len(dup_isa) != 0 else '', end='')
-    for instr_form in dup_isa:
-        print('{}{}{}'.format(BRIGHT_YELLOW, _get_full_instruction_name(instr_form), WHITE))
-    print(
-        'Instruction forms existing in ISA DB but not in uarch DB:\n'
-        if len(only_isa) != 0
-        else '',
-        end='',
-    )
-    for instr_form in only_isa:
-        print('{}{}{}'.format(CYAN, _get_full_instruction_name(instr_form), WHITE))
+    for instr_form in sorted(only_isa, key=lambda i: i['name']):
+        s += '{}{}{}\n'.format(CYAN, _get_full_instruction_name(instr_form), WHITE)
+    return s
 
 
 ###################
@@ -410,6 +546,7 @@ def _print_sanity_report_verbose(
 
 
 def _get_full_instruction_name(instruction_form):
+    """Get full instruction form name/identifier string out of given instruction form."""
     operands = []
     for op in instruction_form['operands']:
         op_attrs = [
@@ -421,16 +558,19 @@ def _get_full_instruction_name(instruction_form):
 
 
 def __represent_none(self, data):
+    """Get YAML None representation."""
     return self.represent_scalar(u'tag:yaml.org,2002:null', u'~')
 
 
 def _create_yaml_object():
+    """Create YAML module with None representation."""
     yaml_obj = ruamel.yaml.YAML()
     yaml_obj.representer.add_representer(type(None), __represent_none)
     return yaml_obj
 
 
 def __dump_data_to_yaml(filepath, data):
+    """Dump data to YAML file at given filepath."""
     # first add 'normal' meta data in the right order (no ordered dict yet)
     meta_data = dict(data)
     del meta_data['instruction_forms']
