@@ -5,19 +5,19 @@ import io
 import os
 import re
 import sys
-from subprocess import call
 
-from osaca.db_interface import sanity_check, import_benchmark_output
+from osaca.db_interface import import_benchmark_output, sanity_check
 from osaca.frontend import Frontend
 from osaca.parser import BaseParser, ParserAArch64v81, ParserX86ATT
-from osaca.semantics import (KernelDG, MachineModel, ArchSemantics,
-                             reduce_to_section)
+from osaca.semantics import (INSTR_FLAGS, ArchSemantics, KernelDG,
+                             MachineModel, reduce_to_section)
 
 MODULE_DATA_DIR = os.path.join(
     os.path.dirname(os.path.split(os.path.abspath(__file__))[0]), 'osaca/data/'
 )
 LOCAL_OSACA_DIR = os.path.join(os.path.expanduser('~') + '/.osaca/')
 DATA_DIR = os.path.join(LOCAL_OSACA_DIR, 'data/')
+SUPPORTED_ARCHS = ['SNB', 'IVB', 'HSW', 'BDW', 'SKX', 'CSX', 'ZEN1', 'ZEN2', 'TX2', 'N1']
 
 
 # Stolen from pip
@@ -48,18 +48,21 @@ def get_version():
     return __find_version('__init__.py')
 
 
-def create_parser():
+def create_parser(parser=None):
     """
     Return argparse parser.
 
+    :param parser: Existing parser object to add the arguments, defaults to `None`
+    :type parser: :class:`~Argparse.ArgumentParser`
     :returns: The newly created :class:`~Argparse.ArgumentParser` object.
     """
     # Create parser
-    parser = argparse.ArgumentParser(
-        description='Analyzes a marked innermost loop snippet for a given architecture type.',
-        epilog='For help, examples, documentation and bug reports go to:\nhttps://github.com'
-        '/RRZE-HPC/OSACA/ | License: AGPLv3',
-    )
+    if not parser:
+        parser = argparse.ArgumentParser(
+            description='Analyzes a marked innermost loop snippet for a given architecture type.',
+            epilog='For help, examples, documentation and bug reports go to:\nhttps://github.com'
+            '/RRZE-HPC/OSACA/ | License: AGPLv3',
+        )
 
     # Add arguments
     parser.add_argument(
@@ -68,13 +71,13 @@ def create_parser():
     parser.add_argument(
         '--arch',
         type=str,
-        help='Define architecture (SNB, IVB, HSW, BDW, SKX, CSX, ZEN1, TX2).',
+        help='Define architecture (SNB, IVB, HSW, BDW, SKX, CSX, ZEN1, ZEN2, TX2, N1).',
     )
     parser.add_argument(
         '--fixed',
         action='store_true',
         help='Run the throughput analysis with fixed probabilities for all suitable ports per '
-        'instruction. Otherwise, OSACA will print out the optimal port utilization for the kernel.'
+        'instruction. Otherwise, OSACA will print the optimal port utilization for the kernel.',
     )
     parser.add_argument(
         '--db-check',
@@ -83,6 +86,13 @@ def create_parser():
         help='Run a sanity check on the by "--arch" specified database. The output depends '
         'on the verbosity level.',
     )
+    parser.add_argument(
+        '--online',
+        dest='internet_check',
+        action='store_true',
+        help='Run sanity check with online DB validation (currently felixcloutier) to see the '
+        'src/dst distribution of the operands. Can be only used in combination with --db-check.',
+    )
     parser.add_argument(
         '--import',
         metavar='MICROBENCH',
@@ -109,9 +119,21 @@ def create_parser():
         help='Output path for .dot file export. If "." is given, the file will be stored as '
         '"./osaca_dg.dot"',
     )
+    parser.add_argument(
+        '--ignore-unknown',
+        dest='ignore_unknown',
+        action='store_true',
+        help='Ignore if instructions cannot be found in the data file and print analysis anyway.',
+    )
     parser.add_argument(
         '--verbose', '-v', action='count', default=0, help='Increases verbosity level.'
     )
+    parser.add_argument(
+        '--out', '-o',
+        default=sys.stdout,
+        type=argparse.FileType('w'),
+        help='Write analysis to this file (default to stdout).'
+    )
     parser.add_argument(
         'file', type=argparse.FileType('r'), help='Path to object (ASM or instruction file).'
     )
@@ -126,10 +148,9 @@ def check_arguments(args, parser):
     :param args: arguments given from :class:`~argparse.ArgumentParser` after parsing
     :param parser: :class:`~argparse.ArgumentParser` object
     """
-    supported_archs = ['SNB', 'IVB', 'HSW', 'BDW', 'SKX', 'CSX', 'ZEN1', 'TX2']
     supported_import_files = ['ibench', 'asmbench']
 
-    if 'arch' in args and args.arch.upper() not in supported_archs:
+    if 'arch' in args and (args.arch is None or args.arch.upper() not in SUPPORTED_ARCHS):
         parser.error(
             'Microarchitecture not supported. Please see --help for all valid architecture codes.'
         )
@@ -138,9 +159,11 @@ def check_arguments(args, parser):
             'Microbenchmark not supported for data import. Please see --help for all valid '
             'microbenchmark codes.'
         )
+    if args.internet_check and not args.check_db:
+        parser.error('--online requires --check-db')
 
 
-def import_data(benchmark_type, arch, filepath):
+def import_data(benchmark_type, arch, filepath, output_file=sys.stdout):
     """
     Imports benchmark results from micro-benchmarks.
 
@@ -150,11 +173,13 @@ def import_data(benchmark_type, arch, filepath):
     :type arch: str
     :param filepath: filepath of the output file"
     :type filepath: str
+    :param output_file: output stream specifying where to write output, defaults to :class:`sys.stdout`
+    :type output_file: stream, optional
     """
     if benchmark_type.lower() == 'ibench':
-        import_benchmark_output(arch, 'ibench', filepath)
+        import_benchmark_output(arch, 'ibench', filepath, output=output_file)
     elif benchmark_type.lower() == 'asmbench':
-        import_benchmark_output(arch, 'asmbench', filepath)
+        import_benchmark_output(arch, 'asmbench', filepath, output=output_file)
     else:
         raise NotImplementedError('This benchmark input variant is not supported.')
 
@@ -165,16 +190,13 @@ def insert_byte_marker(args):
 
     :param args: arguments given from :class:`~argparse.ArgumentParser` after parsing
     """
-    if MachineModel.get_isa_for_arch(args.arch) != 'x86':
-        print('Marker insertion for non-x86 is not yet supported by Kerncraft.', file=sys.stderr)
-        sys.exit(1)
     try:
-        from kerncraft import iaca
+        from kerncraft.incore_model import asm_instrumentation
     except ImportError:
         print(
-            "Module kerncraft not installed. Use 'pip install --user "
-            "kerncraft' for installation.\nFor more information see "
-            "https://github.com/RRZE-HPC/kerncraft",
+            'Module kerncraft not installed. Use \'pip install --user '
+            'kerncraft\' for installation.\nFor more information see '
+            'https://github.com/RRZE-HPC/kerncraft',
             file=sys.stderr,
         )
         sys.exit(1)
@@ -182,11 +204,12 @@ def insert_byte_marker(args):
     assembly = args.file.read()
     unmarked_assembly = io.StringIO(assembly)
     marked_assembly = io.StringIO()
-    iaca.iaca_instrumentation(
+    asm_instrumentation(
         input_file=unmarked_assembly,
         output_file=marked_assembly,
         block_selection='manual',
         pointer_increment='auto_with_manual_fallback',
+        isa=MachineModel.get_isa_for_arch(args.arch),
     )
 
     marked_assembly.seek(0)
@@ -195,16 +218,19 @@ def insert_byte_marker(args):
         f.write(assembly)
 
 
-def inspect(args):
+def inspect(args, output_file=sys.stdout):
     """
     Does the actual throughput and critical path analysis of OSACA and prints it to the
     terminal.
 
     :param args: arguments given from :class:`~argparse.ArgumentParser` after parsing
+    :param output_file: Define the stream for output, defaults to :class:`sys.stdout`
+    :type output_file: stream, optional
     """
     arch = args.arch
     isa = MachineModel.get_isa_for_arch(arch)
     verbose = args.verbose
+    ignore_unknown = args.ignore_unknown
 
     # Read file
     code = args.file.read()
@@ -227,7 +253,12 @@ def inspect(args):
         kernel_graph.export_graph(args.dotpath if args.dotpath != '.' else None)
     # Print analysis
     frontend = Frontend(args.file.name, arch=arch)
-    frontend.print_full_analysis(kernel, kernel_graph, verbose=verbose)
+    print(
+        frontend.full_analysis(
+            kernel, kernel_graph, ignore_unknown=ignore_unknown, verbose=verbose
+        ),
+        file=output_file,
+    )
 
 
 def run(args, output_file=sys.stdout):
@@ -236,23 +267,25 @@ def run(args, output_file=sys.stdout):
 
     :param args: arguments given from :class:`~argparse.ArgumentParser` after parsing
     :param output_file: Define the stream for output, defaults to :class:`sys.stdout`
+    :type output_file: stream, optional
     """
     if args.check_db:
         # Sanity check on DB
         verbose = True if args.verbose > 0 else False
-        sanity_check(args.arch, verbose=verbose)
+        sanity_check(
+            args.arch, verbose=verbose, internet_check=args.internet_check, output_file=output_file
+        )
     elif 'import_data' in args:
         # Import microbench output file into DB
-        import_data(args.import_data, args.arch, args.file.name)
+        import_data(args.import_data, args.arch, args.file.name, output_file=output_file)
     elif args.insert_marker:
         # Try to add IACA marker
         insert_byte_marker(args)
     else:
         # Analyze kernel
-        inspect(args)
+        inspect(args, output_file=output_file)
 
 
-# ---------------------------------------------------
 def get_asm_parser(arch) -> BaseParser:
     """
     Helper function to create the right parser for a specific architecture.
@@ -268,7 +301,16 @@ def get_asm_parser(arch) -> BaseParser:
         return ParserAArch64v81()
 
 
-# ---------------------------------------------------
+def get_unmatched_instruction_ratio(kernel):
+    """Return ratio of unmatched from total instructions in kernel."""
+    unmatched_counter = 0
+    for instruction in kernel:
+        if (
+            INSTR_FLAGS.TP_UNKWN in instruction['flags']
+            and INSTR_FLAGS.LT_UNKWN in instruction['flags']
+        ):
+            unmatched_counter += 1
+    return unmatched_counter / len(kernel)
 
 
 def main():
@@ -276,7 +318,7 @@ def main():
     parser = create_parser()
     args = parser.parse_args()
     check_arguments(args, parser)
-    run(args)
+    run(args, output_file=args.out)
 
 
 if __name__ == '__main__':
