@@ -12,28 +12,30 @@ class ParserAArch64v81(BaseParser):
         self.isa = 'aarch64'
 
     def construct_parser(self):
+        """Create parser for ARM AArch64 ISA."""
         # Comment
         symbol_comment = '//'
         self.comment = pp.Literal(symbol_comment) + pp.Group(
             pp.ZeroOrMore(pp.Word(pp.printables))
         ).setResultsName(self.COMMENT_ID)
         # Define ARM assembly identifier
+        decimal_number = pp.Combine(
+            pp.Optional(pp.Literal('-')) + pp.Word(pp.nums)
+        ).setResultsName('value')
+        hex_number = pp.Combine(pp.Literal('0x') + pp.Word(pp.hexnums)).setResultsName('value')
         relocation = pp.Combine(pp.Literal(':') + pp.Word(pp.alphanums + '_') + pp.Literal(':'))
         first = pp.Word(pp.alphas + '_.', exact=1)
         rest = pp.Word(pp.alphanums + '_.')
         identifier = pp.Group(
             pp.Optional(relocation).setResultsName('relocation')
             + pp.Combine(first + pp.Optional(rest)).setResultsName('name')
-        ).setResultsName('identifier')
+            + pp.Optional(pp.Suppress(pp.Literal('+')) + (hex_number | decimal_number).setResultsName('offset'))
+        ).setResultsName(self.IDENTIFIER_ID)
         # Label
         self.label = pp.Group(
             identifier.setResultsName('name') + pp.Literal(':') + pp.Optional(self.comment)
         ).setResultsName(self.LABEL_ID)
         # Directive
-        decimal_number = pp.Combine(
-            pp.Optional(pp.Literal('-')) + pp.Word(pp.nums)
-        ).setResultsName('value')
-        hex_number = pp.Combine(pp.Literal('0x') + pp.Word(pp.hexnums)).setResultsName('value')
         directive_option = pp.Combine(
             pp.Word(pp.alphas + '#@.%', exact=1)
             + pp.Optional(pp.Word(pp.printables + ' ', excludeChars=','))
@@ -48,6 +50,15 @@ class ParserAArch64v81(BaseParser):
             + commaSeparatedList.setResultsName('parameters')
             + pp.Optional(self.comment)
         ).setResultsName(self.DIRECTIVE_ID)
+        # LLVM-MCA markers
+        self.llvm_markers = pp.Group(
+            pp.Literal('#')
+            + pp.Combine(
+                pp.CaselessLiteral('LLVM-MCA-')
+                + (pp.CaselessLiteral('BEGIN') | pp.CaselessLiteral('END'))
+            )
+            + pp.Optional(self.comment)
+        ).setResultsName(self.COMMENT_ID)
 
         ##############################
         # Instructions
@@ -172,8 +183,9 @@ class ParserAArch64v81(BaseParser):
         Parse line and return instruction form.
 
         :param str line: line of assembly code
-        :param int line_id: default None, identifier of instruction form
-        :return: parsed instruction form
+        :param line_number: identifier of instruction form, defautls to None
+        :type line_number: int, optional
+        :return: `dict` -- parsed asm line (comment, label, directive or instruction form)
         """
         instruction_form = AttrDict(
             {
@@ -195,7 +207,15 @@ class ParserAArch64v81(BaseParser):
             instruction_form[self.COMMENT_ID] = ' '.join(result[self.COMMENT_ID])
         except pp.ParseException:
             pass
-
+        # 1.2 check for llvm-mca marker
+        try:
+            result = self.process_operand(
+                self.llvm_markers.parseString(line, parseAll=True).asDict()
+            )
+            result = AttrDict.convert_dict(result)
+            instruction_form[self.COMMENT_ID] = ' '.join(result[self.COMMENT_ID])
+        except pp.ParseException:
+            pass
         # 2. Parse label
         if result is None:
             try:
@@ -246,6 +266,12 @@ class ParserAArch64v81(BaseParser):
         return instruction_form
 
     def parse_instruction(self, instruction):
+        """
+        Parse instruction in asm line.
+
+        :param str instruction: Assembly line string.
+        :returns: `dict` -- parsed instruction form
+        """
         result = self.instruction_parser.parseString(instruction, parseAll=True).asDict()
         result = AttrDict.convert_dict(result)
         operands = []
@@ -275,25 +301,29 @@ class ParserAArch64v81(BaseParser):
         return return_dict
 
     def process_operand(self, operand):
+        """Post-process operand"""
         # structure memory addresses
         if self.MEMORY_ID in operand:
-            return self.substitute_memory_address(operand[self.MEMORY_ID])
+            return self.process_memory_address(operand[self.MEMORY_ID])
         # structure register lists
         if self.REGISTER_ID in operand and (
             'list' in operand[self.REGISTER_ID] or 'range' in operand[self.REGISTER_ID]
         ):
             # TODO: discuss if ranges should be converted to lists
-            return self.substitute_register_list(operand[self.REGISTER_ID])
+            return self.process_register_list(operand[self.REGISTER_ID])
         if self.REGISTER_ID in operand and operand[self.REGISTER_ID]['name'] == 'sp':
-            return self.substitute_sp_register(operand[self.REGISTER_ID])
+            return self.process_sp_register(operand[self.REGISTER_ID])
         # add value attribute to floating point immediates without exponent
         if self.IMMEDIATE_ID in operand:
-            return self.substitute_immediate(operand[self.IMMEDIATE_ID])
+            return self.process_immediate(operand[self.IMMEDIATE_ID])
         if self.LABEL_ID in operand:
-            return self.substitute_label(operand[self.LABEL_ID])
+            return self.process_label(operand[self.LABEL_ID])
+        if self.IDENTIFIER_ID in operand:
+            return self.process_identifier(operand[self.IDENTIFIER_ID])
         return operand
 
-    def substitute_memory_address(self, memory_address):
+    def process_memory_address(self, memory_address):
+        """Post-process memory address operand"""
         # Remove unnecessarily created dictionary entries during parsing
         offset = None if 'offset' not in memory_address else memory_address['offset']
         base = None if 'base' not in memory_address else memory_address['base']
@@ -315,12 +345,14 @@ class ParserAArch64v81(BaseParser):
             new_dict['post_indexed'] = memory_address['post_indexed']
         return AttrDict({self.MEMORY_ID: new_dict})
 
-    def substitute_sp_register(self, register):
+    def process_sp_register(self, register):
+        """Post-process stack pointer register"""
         reg = register
         reg['prefix'] = 'x'
         return AttrDict({self.REGISTER_ID: reg})
 
-    def substitute_register_list(self, register_list):
+    def process_register_list(self, register_list):
+        """Post-process register lists (e.g., {r0,r3,r5}) and register ranges (e.g., {r0-r7})"""
         # Remove unnecessarily created dictionary entries during parsing
         vlist = []
         dict_name = ''
@@ -336,7 +368,8 @@ class ParserAArch64v81(BaseParser):
         new_dict = AttrDict({dict_name: vlist, 'index': index})
         return AttrDict({self.REGISTER_ID: new_dict})
 
-    def substitute_immediate(self, immediate):
+    def process_immediate(self, immediate):
+        """Post-process immediate operand"""
         dict_name = ''
         if 'identifier' in immediate:
             # actually an identifier, change declaration
@@ -360,12 +393,21 @@ class ParserAArch64v81(BaseParser):
                 {self.IMMEDIATE_ID: AttrDict({'value': immediate[dict_name]['mantissa']})}
             )
 
-    def substitute_label(self, label):
+    def process_label(self, label):
+        """Post-process label asm line"""
         # remove duplicated 'name' level due to identifier
         label['name'] = label['name']['name']
         return AttrDict({self.LABEL_ID: label})
 
+    def process_identifier(self, identifier):
+        """Post-process identifier operand"""
+        # remove value if it consists of symbol+offset
+        if 'value' in identifier:
+            del identifier['value']
+        return AttrDict({self.IDENTIFIER_ID: identifier})
+
     def get_full_reg_name(self, register):
+        """Return one register name string including all attributes"""
         if 'lanes' in register:
             return (
                 register['prefix']
@@ -377,19 +419,21 @@ class ParserAArch64v81(BaseParser):
         return register['prefix'] + str(register['name'])
 
     def normalize_imd(self, imd):
+        """Normalize immediate to decimal based representation"""
         if 'value' in imd:
             if imd['value'].lower().startswith('0x'):
                 # hex, return decimal
                 return int(imd['value'], 16)
             return int(imd['value'], 10)
         elif 'float' in imd:
-            return self.ieee_to_int(imd['float'])
+            return self.ieee_to_float(imd['float'])
         elif 'double' in imd:
-            return self.ieee_to_int(imd['double'])
+            return self.ieee_to_float(imd['double'])
         # identifier
         return imd
 
-    def ieee_to_int(self, ieee_val):
+    def ieee_to_float(self, ieee_val):
+        """Convert IEEE representation to python float"""
         exponent = int(ieee_val['exponent'], 10)
         if ieee_val['e_sign'] == '-':
             exponent *= -1
@@ -399,16 +443,27 @@ class ParserAArch64v81(BaseParser):
         raise NotImplementedError
 
     def is_gpr(self, register):
+        """Check if register is a general purpose register"""
         if register['prefix'] in 'wx':
             return True
         return False
 
     def is_vector_register(self, register):
+        """Check if register is a vector register"""
         if register['prefix'] in 'bhsdqv':
             return True
         return False
 
+    def is_flag_dependend_of(self, flag_a, flag_b):
+        """Check if ``flag_a`` is dependent on ``flag_b``"""
+        # we assume flags are independent of each other, e.g., CF can be read while ZF gets written
+        # TODO validate this assumption
+        if flag_a.name == flag_b.name:
+            return True
+        return False
+
     def is_reg_dependend_of(self, reg_a, reg_b):
+        """Check if ``reg_a`` is dependent on ``reg_b``"""
         prefixes_gpr = 'wx'
         prefixes_vec = 'bhsdqv'
         if reg_a['name'] == reg_b['name']:
@@ -419,4 +474,5 @@ class ParserAArch64v81(BaseParser):
         return False
 
     def get_reg_type(self, register):
+        """Get register type"""
         return register['prefix']
