@@ -1,5 +1,7 @@
 #!/usr/bin/env python3
 
+import string
+
 import pyparsing as pp
 
 from osaca.parser import AttrDict, BaseParser
@@ -11,36 +13,56 @@ class ParserX86ATT(BaseParser):
         self.isa = 'x86'
 
     def construct_parser(self):
+        """Create parser for ARM AArch64 ISA."""
         decimal_number = pp.Combine(
             pp.Optional(pp.Literal('-')) + pp.Word(pp.nums)
         ).setResultsName('value')
         hex_number = pp.Combine(pp.Literal('0x') + pp.Word(pp.hexnums)).setResultsName('value')
-        # Comment
-        symbol_comment = '#'
-        self.comment = pp.Literal(symbol_comment) + pp.Group(
+        # Comment - either '#' or '//' (icc)
+        self.comment = (pp.Literal('#') | pp.Literal('//')) + pp.Group(
             pp.ZeroOrMore(pp.Word(pp.printables))
         ).setResultsName(self.COMMENT_ID)
         # Define x86 assembly identifier
+        relocation = pp.Combine(pp.Literal('@') + pp.Word(pp.alphas))
         id_offset = pp.Word(pp.nums) + pp.Suppress(pp.Literal('+'))
         first = pp.Word(pp.alphas + '_.', exact=1)
-        rest = pp.Word(pp.alphanums + '$_.')
+        rest = pp.Word(pp.alphanums + '$_.+-')
         identifier = pp.Group(
             pp.Optional(id_offset).setResultsName('offset')
             + pp.Combine(first + pp.Optional(rest)).setResultsName('name')
+            + pp.Optional(relocation).setResultsName('relocation')
         ).setResultsName('identifier')
         # Label
+        rest = pp.Word(pp.alphanums + '$_.+-()')
+        label_identifier = pp.Group(
+            pp.Optional(id_offset).setResultsName('offset')
+            + pp.Combine(first + pp.Optional(rest)).setResultsName('name')
+            + pp.Optional(relocation).setResultsName('relocation')
+        ).setResultsName('identifier')
+        numeric_identifier = pp.Group(
+            pp.Word(pp.nums).setResultsName('name')
+            + pp.Optional(pp.oneOf('b f', caseless=True).setResultsName('suffix'))
+        ).setResultsName('identifier')
         self.label = pp.Group(
-            identifier.setResultsName('name') + pp.Literal(':') + pp.Optional(self.comment)
+            (label_identifier | numeric_identifier).setResultsName('name')
+            + pp.Literal(':')
+            + pp.Optional(self.comment)
         ).setResultsName(self.LABEL_ID)
-        # Register: pp.Regex('^%[0-9a-zA-Z]+,?')
+        # Register: pp.Regex('^%[0-9a-zA-Z]+{}{z},?')
         self.register = pp.Group(
             pp.Literal('%')
             + pp.Word(pp.alphanums).setResultsName('name')
+            + pp.Optional(pp.Literal('(') + pp.Word(pp.nums) + pp.Literal(')'))
             + pp.Optional(
                 pp.Literal('{')
-                + pp.Literal('%')
+                + pp.Optional(pp.Suppress(pp.Literal('%')))
                 + pp.Word(pp.alphanums).setResultsName('mask')
                 + pp.Literal('}')
+                + pp.Optional(
+                    pp.Suppress(pp.Literal('{'))
+                    + pp.Literal('z').setResultsName('zeroing')
+                    + pp.Suppress(pp.Literal('}'))
+                )
             )
         ).setResultsName(self.REGISTER_ID)
         # Immediate: pp.Regex('^\$(-?[0-9]+)|(0x[0-9a-fA-F]+),?')
@@ -48,39 +70,70 @@ class ParserX86ATT(BaseParser):
         immediate = pp.Group(
             pp.Literal(symbol_immediate) + (hex_number | decimal_number | identifier)
         ).setResultsName(self.IMMEDIATE_ID)
-        # Memory: offset(base, index, scale)
+
+        # Memory preparations
         offset = pp.Group(identifier | hex_number | decimal_number).setResultsName(
             self.IMMEDIATE_ID
         )
         scale = pp.Word('1248', exact=1)
+        # Segment register extension
+        segment_extension = (
+            hex_number
+            ^ pp.Word(pp.nums)
+            ^ pp.Group(
+                pp.Optional(offset.setResultsName('offset'))
+                + pp.Literal('(')
+                + pp.Optional(self.register.setResultsName('base'))
+                + pp.Optional(pp.Suppress(pp.Literal(',')))
+                + pp.Optional(self.register.setResultsName('index'))
+                + pp.Optional(pp.Suppress(pp.Literal(',')))
+                + pp.Optional(scale.setResultsName('scale'))
+                + pp.Literal(')')
+            )
+        )
+        memory_segmentation = (
+            self.register.setResultsName('base')
+            + pp.Literal(':')
+            + segment_extension.setResultsName(self.SEGMENT_EXT_ID)
+        )
+        # Memory: offset | seg:seg_ext | offset(base, index, scale){mask}
         memory = pp.Group(
-            pp.Optional(offset.setResultsName('offset'))
-            + pp.Literal('(')
-            + pp.Optional(self.register.setResultsName('base'))
-            + pp.Optional(pp.Suppress(pp.Literal(',')))
-            + pp.Optional(self.register.setResultsName('index'))
-            + pp.Optional(pp.Suppress(pp.Literal(',')))
-            + pp.Optional(scale.setResultsName('scale'))
-            + pp.Literal(')')
+            (
+                pp.Optional(pp.Suppress(pp.Literal('*')))
+                + pp.Optional(offset.setResultsName('offset'))
+                + pp.Literal('(')
+                + pp.Optional(self.register.setResultsName('base'))
+                + pp.Optional(pp.Suppress(pp.Literal(',')))
+                + pp.Optional(self.register.setResultsName('index'))
+                + pp.Optional(pp.Suppress(pp.Literal(',')))
+                + pp.Optional(scale.setResultsName('scale'))
+                + pp.Literal(')')
+                + pp.Optional(
+                    pp.Literal('{')
+                    + pp.Optional(pp.Suppress(pp.Literal('%')))
+                    + pp.Word(pp.alphanums).setResultsName('mask')
+                    + pp.Literal('}')
+                )
+            )
+            | memory_segmentation
+            | (hex_number | pp.Word(pp.nums)).setResultsName('offset')
         ).setResultsName(self.MEMORY_ID)
 
         # Directive
-        directive_option = pp.Combine(
-            pp.Word('#@.', exact=1) + pp.Word(pp.printables, excludeChars=',')
-        )
+        # parameter can be any quoted string or sequence of characters besides '#' (for comments)
+        # or ',' (parameter delimiter)
         directive_parameter = (
             pp.quotedString
-            | directive_option
-            | identifier
-            | hex_number
-            | decimal_number
-            | self.register
+            ^ (
+                pp.Word(pp.printables, excludeChars=',#')
+                + pp.Optional(pp.Suppress(pp.Literal(',')))
+            )
+            ^ pp.Suppress(pp.Literal(','))
         )
-        commaSeparatedList = pp.delimitedList(pp.Optional(directive_parameter), delim=',')
         self.directive = pp.Group(
             pp.Literal('.')
             + pp.Word(pp.alphanums + '_').setResultsName('name')
-            + commaSeparatedList.setResultsName('parameters')
+            + pp.ZeroOrMore(directive_parameter).setResultsName('parameters')
             + pp.Optional(self.comment)
         ).setResultsName(self.DIRECTIVE_ID)
 
@@ -90,7 +143,9 @@ class ParserX86ATT(BaseParser):
             pp.alphanums
         ).setResultsName('mnemonic')
         # Combine to instruction form
-        operand_first = pp.Group(self.register ^ immediate ^ memory ^ identifier)
+        operand_first = pp.Group(
+            self.register ^ immediate ^ memory ^ identifier ^ numeric_identifier
+        )
         operand_rest = pp.Group(self.register ^ immediate ^ memory)
         self.instruction_parser = (
             mnemonic
@@ -105,6 +160,7 @@ class ParserX86ATT(BaseParser):
         )
 
     def parse_register(self, register_string):
+        """Parse register string"""
         try:
             return self.process_operand(
                 self.register.parseString(register_string, parseAll=True).asDict()
@@ -117,8 +173,9 @@ class ParserX86ATT(BaseParser):
         Parse line and return instruction form.
 
         :param str line: line of assembly code
-        :param int line_id: default None, identifier of instruction form
-        :return: parsed instruction form
+        :param line_number: default None, identifier of instruction form
+        :type line_number: int, optional
+        :return: ``dict`` -- parsed asm line (comment, label, directive or instruction form)
         """
         instruction_form = AttrDict(
             {
@@ -189,6 +246,12 @@ class ParserX86ATT(BaseParser):
         return instruction_form
 
     def parse_instruction(self, instruction):
+        """
+        Parse instruction in asm line.
+
+        :param str instruction: Assembly line string.
+        :returns: `dict` -- parsed instruction form
+        """
         result = self.instruction_parser.parseString(instruction, parseAll=True).asDict()
         result = AttrDict.convert_dict(result)
         operands = []
@@ -217,30 +280,49 @@ class ParserX86ATT(BaseParser):
         return return_dict
 
     def process_operand(self, operand):
+        """Post-process operand"""
         # For the moment, only used to structure memory addresses
         if self.MEMORY_ID in operand:
-            return self.substitute_memory_address(operand[self.MEMORY_ID])
+            return self.process_memory_address(operand[self.MEMORY_ID])
         if self.IMMEDIATE_ID in operand:
-            return self.substitue_immediate(operand[self.IMMEDIATE_ID])
+            return self.process_immediate(operand[self.IMMEDIATE_ID])
         if self.LABEL_ID in operand:
-            return self.substitute_label(operand[self.LABEL_ID])
+            return self.process_label(operand[self.LABEL_ID])
+        if self.DIRECTIVE_ID in operand:
+            return self.process_directive(operand[self.DIRECTIVE_ID])
         return operand
 
-    def substitute_memory_address(self, memory_address):
+    def process_directive(self, directive):
+        directive_new = {'name': directive['name'], 'parameters': []}
+        if 'parameters' in directive:
+            directive_new['parameters'] = directive['parameters']
+        if 'comment' in directive:
+            directive_new['comment'] = directive['comment']
+        return AttrDict({self.DIRECTIVE_ID: directive_new})
+
+    def process_memory_address(self, memory_address):
+        """Post-process memory address operand"""
         # Remove unecessarily created dictionary entries during memory address parsing
         offset = None if 'offset' not in memory_address else memory_address['offset']
         base = None if 'base' not in memory_address else memory_address['base']
         index = None if 'index' not in memory_address else memory_address['index']
         scale = 1 if 'scale' not in memory_address else int(memory_address['scale'])
+        if isinstance(offset, str) and base is None and index is None:
+            offset = {'value': offset}
         new_dict = AttrDict({'offset': offset, 'base': base, 'index': index, 'scale': scale})
+        # Add segmentation extension if existing
+        if self.SEGMENT_EXT_ID in memory_address:
+            new_dict[self.SEGMENT_EXT_ID] = memory_address[self.SEGMENT_EXT_ID]
         return AttrDict({self.MEMORY_ID: new_dict})
 
-    def substitute_label(self, label):
+    def process_label(self, label):
+        """Post-process label asm line"""
         # remove duplicated 'name' level due to identifier
-        label['name'] = label['name']['name']
+        label['name'] = label['name'][0]['name']
         return AttrDict({self.LABEL_ID: label})
 
-    def substitue_immediate(self, immediate):
+    def process_immediate(self, immediate):
+        """Post-process immediate operand"""
         if 'identifier' in immediate:
             # actually an identifier, change declaration
             return immediate
@@ -248,10 +330,12 @@ class ParserX86ATT(BaseParser):
         return AttrDict({self.IMMEDIATE_ID: immediate})
 
     def get_full_reg_name(self, register):
+        """Return one register name string including all attributes"""
         # nothing to do
         return register['name']
 
     def normalize_imd(self, imd):
+        """Normalize immediate to decimal based representation"""
         if 'value' in imd:
             if imd['value'].lower().startswith('0x'):
                 # hex, return decimal
@@ -260,7 +344,16 @@ class ParserX86ATT(BaseParser):
         # identifier
         return imd
 
+    def is_flag_dependend_of(self, flag_a, flag_b):
+        """Check if ``flag_a`` is dependent on ``flag_b``"""
+        # we assume flags are independent of each other, e.g., CF can be read while ZF gets written
+        # TODO validate this assumption
+        if flag_a.name == flag_b.name:
+            return True
+        return False
+
     def is_reg_dependend_of(self, reg_a, reg_b):
+        """Check if ``reg_a`` is dependent on ``reg_b``"""
         # Check if they are the same registers
         if reg_a.name == reg_b.name:
             return True
@@ -304,11 +397,15 @@ class ParserX86ATT(BaseParser):
         return False
 
     def is_basic_gpr(self, register):
+        """Check if register is a basic general purpose register (ebi, rax, ...)"""
         if any(char.isdigit() for char in register['name']):
             return False
         return True
 
     def is_gpr(self, register):
+        """Check if register is a general purpose register"""
+        if register is None:
+            return False
         gpr_parser = (
             pp.CaselessLiteral('R')
             + pp.Word(pp.nums).setResultsName('id')
@@ -324,13 +421,19 @@ class ParserX86ATT(BaseParser):
                 return False
 
     def is_vector_register(self, register):
-        if len(register['name']) > 2 and register['name'][1:3].lower() == 'mm':
+        """Check if register is a vector register"""
+        if register is None:
+            return False
+        if register['name'].rstrip(string.digits).lower() in ['mm', 'xmm', 'ymm', 'zmm']:
             return True
         return False
 
     def get_reg_type(self, register):
+        """Ger register type"""
+        if register is None:
+            return False
         if self.is_gpr(register):
             return 'gpr'
         elif self.is_vector_register(register):
-            return register['name'][:3].lower()
+            return register['name'].rstrip(string.digits).lower()
         raise ValueError
