@@ -3,8 +3,17 @@ from collections import OrderedDict
 
 from osaca.parser import ParserAArch64v81, ParserX86ATT, get_parser
 
+COMMENT_MARKER = {'start': 'OSACA-BEGIN', 'end': 'OSACA-END'}
+
 
 def reduce_to_section(kernel, isa):
+    """
+    Finds OSACA markers in given kernel and returns marked section
+
+    :param list kernel: kernel to check
+    :param str isa: ISA of given kernel
+    :returns: `list` -- marked section of kernel as list of instruction forms
+    """
     isa = isa.lower()
     if isa == 'x86':
         start, end = find_marked_kernel_x86ATT(kernel)
@@ -13,28 +22,52 @@ def reduce_to_section(kernel, isa):
     else:
         raise ValueError('ISA not supported.')
     if start == -1:
-        raise LookupError('Could not find START MARKER. Make sure it is inserted!')
-    if end == -1:
-        raise LookupError('Could not find END MARKER. Make sure it is inserted!')
+        start = 0
     return kernel[start:end]
 
 
 def find_marked_kernel_AArch64(lines):
+    """
+    Find marked section for AArch64
+
+    :param list lines: kernel
+    :returns: `tuple of int` -- start and end line of marked section
+    """
     nop_bytes = ['213', '3', '32', '31']
     return find_marked_section(
-        lines, ParserAArch64v81(), ['mov'], 'x1', [111, 222], nop_bytes, reverse=True
+        lines,
+        ParserAArch64v81(),
+        ['mov'],
+        'x1',
+        [111, 222],
+        nop_bytes,
+        reverse=True,
+        comments=COMMENT_MARKER,
     )
 
 
 def find_marked_kernel_x86ATT(lines):
+    """
+    Find marked section for x86
+
+    :param list lines: kernel
+    :returns: `tuple of int` -- start and end line of marked section
+    """
     nop_bytes = ['100', '103', '144']
     return find_marked_section(
-        lines, ParserX86ATT(), ['mov', 'movl'], 'ebx', [111, 222], nop_bytes
+        lines,
+        ParserX86ATT(),
+        ['mov', 'movl'],
+        'ebx',
+        [111, 222],
+        nop_bytes,
+        comments=COMMENT_MARKER,
     )
 
 
-def get_marker(isa):
+def get_marker(isa, comment=""):
     """Return tuple of start and end marker lines."""
+    isa = isa.lower()
     if isa == 'x86':
         start_marker_raw = (
             'movl      $111, %ebx # OSACA START MARKER\n'
@@ -42,17 +75,21 @@ def get_marker(isa):
             '.byte     103        # OSACA START MARKER\n'
             '.byte     144        # OSACA START MARKER\n'
         )
+        if comment:
+            start_marker_raw += "# {}\n".format(comment)
         end_marker_raw = (
             'movl      $222, %ebx # OSACA END MARKER\n'
             '.byte     100        # OSACA END MARKER\n'
             '.byte     103        # OSACA END MARKER\n'
             '.byte     144        # OSACA END MARKER\n'
         )
-    elif isa == 'AArch64':
+    elif isa == 'aarch64':
         start_marker_raw = (
             'mov       x1, #111    // OSACA START MARKER\n'
             '.byte     213,3,32,31 // OSACA START MARKER\n'
         )
+        if comment:
+            start_marker_raw += "// {}\n".format(comment)
         # After loop
         end_marker_raw = (
             'mov       x1, #222    // OSACA END MARKER\n'
@@ -66,13 +103,40 @@ def get_marker(isa):
     return start_marker, end_marker
 
 
-def find_marked_section(lines, parser, mov_instr, mov_reg, mov_vals, nop_bytes, reverse=False):
+def find_marked_section(
+    lines, parser, mov_instr, mov_reg, mov_vals, nop_bytes, reverse=False, comments=None
+):
+    """
+    Return indexes of marked section
+
+    :param list lines: kernel
+    :param parser: parser to use for checking
+    :type parser: :class:`~parser.BaseParser`
+    :param mov_instr: all MOV instruction possible for the marker
+    :type mov_instr: `list of str`
+    :param mov_reg: register used for the marker
+    :type mov_reg: `str`
+    :param mov_vals: values needed to be moved to ``mov_reg`` for valid marker
+    :type mov_vals: `list of int`
+    :param nop_bytes: bytes representing opcode of NOP
+    :type nop_bytes: `list of int`
+    :param reverse: indicating if ISA syntax requires reverse operand order, defaults to `False`
+    :type reverse: boolean, optional
+    :param comments: dictionary with start and end markers in comment format, defaults to None
+    :type comments: dict, optional
+    :returns: `tuple of int` -- start and end line of marked section
+    """
     # TODO match to instructions returned by get_marker
     index_start = -1
     index_end = -1
     for i, line in enumerate(lines):
         try:
-            if line.instruction in mov_instr and lines[i + 1].directive is not None:
+            if line.instruction is None and comments is not None and line.comment is not None:
+                if comments['start'] == line.comment:
+                    index_start = i + 1
+                elif comments['end'] == line.comment:
+                    index_end = i
+            elif line.instruction in mov_instr and lines[i + 1].directive is not None:
                 source = line.operands[0 if not reverse else 1]
                 destination = line.operands[1 if not reverse else 0]
                 # instruction pair matches, check for operands
@@ -106,6 +170,7 @@ def find_marked_section(lines, parser, mov_instr, mov_reg, mov_vals, nop_bytes,
 
 
 def match_bytes(lines, index, byte_list):
+    """Match bytes directives of markers"""
     # either all bytes are in one line or in separate ones
     extracted_bytes = []
     line_count = 0
@@ -117,7 +182,7 @@ def match_bytes(lines, index, byte_list):
         line_count += 1
         extracted_bytes += lines[index].directive.parameters
         index += 1
-    if extracted_bytes[0:len(byte_list)] == byte_list:
+    if extracted_bytes[0 : len(byte_list)] == byte_list:
         return True, line_count
     return False, -1
 
@@ -152,7 +217,7 @@ def find_jump_labels(lines):
         if all(
             [
                 l['instruction'].startswith('.')
-                for l in lines[labels[label][0]:labels[label][1]]
+                for l in lines[labels[label][0] : labels[label][1]]
                 if l['instruction'] is not None
             ]
         ):
@@ -175,8 +240,8 @@ def find_basic_blocks(lines):
     # an instruction referencing a valid jump label
     blocks = OrderedDict()
     for label, label_line_idx in valid_jump_labels.items():
-        blocks[label] = []
-        for line in lines[label_line_idx + 1:]:
+        blocks[label] = [lines[label_line_idx]]
+        for line in lines[label_line_idx + 1 :]:
             terminate = False
             blocks[label].append(line)
             # Find end of block by searching for references to valid jump labels
@@ -204,8 +269,8 @@ def find_basic_loop_bodies(lines):
     # an instruction referencing a valid jump label
     loop_bodies = OrderedDict()
     for label, label_line_idx in valid_jump_labels.items():
-        current_block = []
-        for line in lines[label_line_idx + 1:]:
+        current_block = [lines[label_line_idx]]
+        for line in lines[label_line_idx + 1 :]:
             terminate = False
             current_block.append(line)
             # Find end of block by searching for references to valid jump labels
