@@ -16,9 +16,9 @@ class TestMarkerUtils(unittest.TestCase):
     def setUpClass(self):
         self.parser_AArch = ParserAArch64v81()
         self.parser_x86 = ParserX86ATT()
-        with open(self._find_file('triad-arm-iaca.s')) as f:
+        with open(self._find_file('triad_arm_iaca.s')) as f:
             triad_code_arm = f.read()
-        with open(self._find_file('triad-x86-iaca.s')) as f:
+        with open(self._find_file('triad_x86_iaca.s')) as f:
             triad_code_x86 = f.read()
         self.parsed_AArch = self.parser_AArch.parse_file(triad_code_arm)
         self.parsed_x86 = self.parser_x86.parse_file(triad_code_x86)
@@ -216,24 +216,6 @@ class TestMarkerUtils(unittest.TestCase):
         kernel_start = len(list(filter(None, (prologue + mov_start + bytes_line).split('\n'))))
         self.assertEqual(test_kernel, [])
 
-        # no start marker
-        code_no_start = prologue + bytes_line + kernel + mov_end + bytes_line + epilogue
-        no_start_parsed = self.parser_AArch.parse_file(code_no_start)
-        with self.assertRaises(LookupError):
-            reduce_to_section(no_start_parsed, 'AArch64')
-
-        # no end marker
-        code_no_end = prologue + mov_start + bytes_line + kernel + mov_end + epilogue
-        no_end_parsed = self.parser_AArch.parse_file(code_no_end)
-        with self.assertRaises(LookupError):
-            reduce_to_section(no_end_parsed, 'AArch64')
-
-        # no marker at all
-        code_no_marker = prologue + kernel + epilogue
-        no_marker_parsed = self.parser_AArch.parse_file(code_no_marker)
-        with self.assertRaises(LookupError):
-            reduce_to_section(no_marker_parsed, 'AArch64')
-
     def test_marker_special_cases_x86(self):
         bytes_line = '.byte     100\n.byte     103\n.byte     144\n'
         mov_start = 'movl     $111, %ebx\n'
@@ -275,24 +257,6 @@ class TestMarkerUtils(unittest.TestCase):
         kernel_start = len(list(filter(None, (prologue + mov_start + bytes_line).split('\n'))))
         self.assertEqual(test_kernel, [])
 
-        # no start marker
-        code_no_start = prologue + bytes_line + kernel + mov_end + bytes_line + epilogue
-        no_start_parsed = self.parser_x86.parse_file(code_no_start)
-        with self.assertRaises(LookupError):
-            reduce_to_section(no_start_parsed, 'x86')
-
-        # no end marker
-        code_no_end = prologue + mov_start + bytes_line + kernel + mov_end + epilogue
-        no_end_parsed = self.parser_x86.parse_file(code_no_end)
-        with self.assertRaises(LookupError):
-            reduce_to_section(no_end_parsed, 'x86')
-
-        # no marker at all
-        code_no_marker = prologue + kernel + epilogue
-        no_marker_parsed = self.parser_x86.parse_file(code_no_marker)
-        with self.assertRaises(LookupError):
-            reduce_to_section(no_marker_parsed, 'x86')
-
     def test_find_jump_labels(self):
         self.assertEqual(find_jump_labels(self.parsed_x86),
                          OrderedDict([('.LFB24', 10), ('.L4', 65), ('.L3', 79), ('.L2', 102),
@@ -317,36 +281,36 @@ class TestMarkerUtils(unittest.TestCase):
         self.assertEqual(
             [(k, v[0]['line_number'], v[-1]['line_number'])
              for k, v in find_basic_blocks(self.parsed_x86).items()],
-            [('.LFB24', 12, 56), ('.L4', 67, 74), ('.L3', 81, 89), ('.L2', 104, 112),
-             ('.L13', 113, 121), ('.L12', 122, 125), ('.L6', 134, 135), ('.L10', 147, 154),
-             ('.L9', 163, 170), ('.L8', 185, 187), ('.L15', 254, 256), ('.L26', 258, 259),
-             ('.L14', 261, 262), ('.LFB25', 279, 290), ('.L28', 291, 300)])
+            [('.LFB24', 11, 56), ('.L4', 66, 74), ('.L3', 80, 89), ('.L2', 103, 112),
+             ('.L13', 112, 121), ('.L12', 121, 125), ('.L6', 133, 135), ('.L10', 146, 154),
+             ('.L9', 162, 170), ('.L8', 184, 187), ('.L15', 253, 256), ('.L26', 257, 259),
+             ('.L14', 260, 262), ('.LFB25', 278, 290), ('.L28', 290, 300)])
 
         self.assertEqual(
             [(k, v[0]['line_number'], v[-1]['line_number'])
              for k, v in find_basic_blocks(self.parsed_AArch).items()],
-            [('triad', 20, 64), ('.LBB0_3', 73, 77), ('.LBB0_4', 78, 83), ('.LBB0_5', 86, 89),
-             ('.LBB0_7', 94, 95), ('.LBB0_8', 97, 105), ('.LBB0_9', 108, 114),
-             ('.LBB0_11', 120, 134), ('.LBB0_12', 135, 173), ('.LBB0_14', 179, 191),
-             ('.LBB0_15', 192, 205), ('.LBB0_16', 207, 208), ('.LBB0_17', 210, 222),
-             ('.LBB0_18', 223, 228), ('.LBB0_19', 230, 261), ('.LBB0_20', 262, 269),
-             ('.LBB0_22', 274, 280), ('.LBB0_24', 285, 286), ('.LBB0_26', 292, 293),
-             ('.LBB0_28', 300, 307), ('.LBB0_29', 308, 444), ('.LBB0_31', 450, 459),
-             ('.LBB0_32', 460, 480), ('.LBB0_33', 482, 484), ('.LBB0_34', 486, 494),
-             ('.LBB0_35', 495, 504), ('.LBB0_36', 506, 508), ('.LBB0_37', 510, 518),
-             ('.LBB0_38', 520, 568), ('main', 576, 590)])
+            [('triad', 19, 64), ('.LBB0_3', 72, 77), ('.LBB0_4', 77, 83), ('.LBB0_5', 85, 89),
+             ('.LBB0_7', 93, 95), ('.LBB0_8', 96, 105), ('.LBB0_9', 107, 114),
+             ('.LBB0_11', 119, 134), ('.LBB0_12', 134, 173), ('.LBB0_14', 178, 191),
+             ('.LBB0_15', 191, 205), ('.LBB0_16', 206, 208), ('.LBB0_17', 209, 222),
+             ('.LBB0_18', 222, 228), ('.LBB0_19', 229, 261), ('.LBB0_20', 261, 269),
+             ('.LBB0_22', 273, 280), ('.LBB0_24', 284, 286), ('.LBB0_26', 291, 293),
+             ('.LBB0_28', 299, 307), ('.LBB0_29', 307, 444), ('.LBB0_31', 449, 459),
+             ('.LBB0_32', 459, 480), ('.LBB0_33', 481, 484), ('.LBB0_34', 485, 494),
+             ('.LBB0_35', 494, 504), ('.LBB0_36', 505, 508), ('.LBB0_37', 509, 518),
+             ('.LBB0_38', 519, 568), ('main', 575, 590)])
 
     def test_find_basic_loop_body(self):
         self.assertEqual(
             [(k, v[0]['line_number'], v[-1]['line_number'])
              for k, v in find_basic_loop_bodies(self.parsed_x86).items()],
-            [('.L4', 67, 74), ('.L10', 147, 154), ('.L28', 291, 300)])
+            [('.L4', 66, 74), ('.L10', 146, 154), ('.L28', 290, 300)])
 
         self.assertEqual(
             [(k, v[0]['line_number'], v[-1]['line_number'])
              for k, v in find_basic_loop_bodies(self.parsed_AArch).items()],
-            [('.LBB0_12', 135, 173), ('.LBB0_15', 192, 205), ('.LBB0_18', 223, 228),
-             ('.LBB0_29', 308, 444), ('.LBB0_32', 460, 480), ('.LBB0_35', 495, 504)])
+            [('.LBB0_12', 134, 173), ('.LBB0_15', 191, 205), ('.LBB0_18', 222, 228),
+             ('.LBB0_29', 307, 444), ('.LBB0_32', 459, 480), ('.LBB0_35', 494, 504)])
 
     ##################
     # Helper functions
