@@ -15,7 +15,7 @@ class TestParserX86ATT(unittest.TestCase):
     @classmethod
     def setUpClass(self):
         self.parser = ParserX86ATT()
-        with open(self._find_file('triad-x86-iaca.s')) as f:
+        with open(self._find_file('triad_x86_iaca.s')) as f:
             self.triad_code = f.read()
 
     ##################
@@ -45,6 +45,31 @@ class TestParserX86ATT(unittest.TestCase):
         self.assertEqual(len(self._get_directive(self.parser, '\t.text').parameters), 0)
         self.assertEqual(self._get_directive(self.parser, '\t.align\t16,0x90').name, 'align')
         self.assertEqual(len(self._get_directive(self.parser, '\t.align\t16,0x90').parameters), 2)
+        self.assertEqual(len(self._get_directive(self.parser, '.text').parameters), 0)
+        self.assertEqual(
+            len(self._get_directive(self.parser, '.file\t1 "path/to/file.c"').parameters), 2
+        )
+        self.assertEqual(
+            self._get_directive(self.parser, '.file\t1 "path/to/file.c"').parameters[1],
+            '"path/to/file.c"',
+        )
+        self.assertEqual(
+            self._get_directive(self.parser, '\t.set\tL$set$0,LECIE1-LSCIE1').parameters,
+            ['L$set$0', 'LECIE1-LSCIE1'],
+        )
+        self.assertEqual(
+            self._get_directive(
+                self.parser,
+                '\t.section __TEXT,__eh_frame,coalesced,no_toc+strip_static_syms+live_support',
+            ).parameters,
+            ['__TEXT', '__eh_frame', 'coalesced', 'no_toc+strip_static_syms+live_support'],
+        )
+        self.assertEqual(
+            self._get_directive(
+                self.parser, '\t.section\t__TEXT,__literal16,16byte_literals'
+            ).parameters,
+            ['__TEXT', '__literal16', '16byte_literals'],
+        )
         self.assertEqual(
             self._get_directive(self.parser, '\t.align\t16,0x90').parameters[1], '0x90'
         )
@@ -172,9 +197,7 @@ class TestParserX86ATT(unittest.TestCase):
                         'scale': 1,
                     }
                 },
-                {
-                    'register': {'name': 'ecx'}
-                }
+                {'register': {'name': 'ecx'}},
             ],
             'directive': None,
             'comment': '12.9',
