@@ -5,13 +5,15 @@ Unit tests for Semantic Analysis
 
 import os
 import unittest
+from copy import deepcopy
 from subprocess import call
 
 import networkx as nx
 
+from osaca.osaca import get_unmatched_instruction_ratio
 from osaca.parser import AttrDict, ParserAArch64v81, ParserX86ATT
-from osaca.semantics import (INSTR_FLAGS, KernelDG, MachineModel,
-                             ArchSemantics)
+from osaca.semantics import (INSTR_FLAGS, ArchSemantics, KernelDG,
+                             MachineModel, reduce_to_section)
 
 
 class TestSemanticTools(unittest.TestCase):
@@ -29,12 +31,14 @@ class TestSemanticTools(unittest.TestCase):
         # set up parser and kernels
         self.parser_x86 = ParserX86ATT()
         self.parser_AArch64 = ParserAArch64v81()
-        with open(self._find_file('kernel-x86.s')) as f:
+        with open(self._find_file('kernel_x86.s')) as f:
             self.code_x86 = f.read()
-        with open(self._find_file('kernel-AArch64.s')) as f:
+        with open(self._find_file('kernel_aarch64.s')) as f:
             self.code_AArch64 = f.read()
-        self.kernel_x86 = self.parser_x86.parse_file(self.code_x86)
-        self.kernel_AArch64 = self.parser_AArch64.parse_file(self.code_AArch64)
+        self.kernel_x86 = reduce_to_section(self.parser_x86.parse_file(self.code_x86), 'x86')
+        self.kernel_AArch64 = reduce_to_section(
+            self.parser_AArch64.parse_file(self.code_AArch64), 'aarch64'
+        )
 
         # set up machine models
         self.machine_model_csx = MachineModel(
@@ -70,6 +74,128 @@ class TestSemanticTools(unittest.TestCase):
         except ValueError:
             self.fail()
 
+    def test_machine_model_various_functions(self):
+        # check dummy MachineModel creation
+        try:
+            MachineModel(isa='x86')
+            MachineModel(isa='aarch64')
+        except ValueError:
+            self.fail()
+        test_mm_x86 = MachineModel(path_to_yaml=self._find_file('test_db_x86.yml'))
+        test_mm_arm = MachineModel(path_to_yaml=self._find_file('test_db_aarch64.yml'))
+
+        # test get_instruction without mnemonic
+        self.assertIsNone(test_mm_x86.get_instruction(None, []))
+        self.assertIsNone(test_mm_arm.get_instruction(None, []))
+
+        # test dict DB creation
+        test_mm_x86._data['instruction_dict'] = test_mm_x86._convert_to_dict(
+            test_mm_x86._data['instruction_forms']
+        )
+        test_mm_arm._data['instruction_dict'] = test_mm_arm._convert_to_dict(
+            test_mm_arm._data['instruction_forms']
+        )
+        # test get_instruction from dict DB
+        self.assertIsNone(test_mm_x86.get_instruction_from_dict(None, []))
+        self.assertIsNone(test_mm_arm.get_instruction_from_dict(None, []))
+        self.assertIsNone(test_mm_x86.get_instruction_from_dict('NOT_IN_DB', []))
+        self.assertIsNone(test_mm_arm.get_instruction_from_dict('NOT_IN_DB', []))
+        name_x86_1 = 'vaddpd'
+        operands_x86_1 = [
+            {'class': 'register', 'name': 'xmm'},
+            {'class': 'register', 'name': 'xmm'},
+            {'class': 'register', 'name': 'xmm'},
+        ]
+        instr_form_x86_1 = test_mm_x86.get_instruction_from_dict(name_x86_1, operands_x86_1)
+        self.assertEqual(instr_form_x86_1, test_mm_x86.get_instruction(name_x86_1, operands_x86_1))
+        self.assertEqual(
+            test_mm_x86.get_instruction_from_dict('jg', [{'class': 'identifier'}]),
+            test_mm_x86.get_instruction('jg', [{'class': 'identifier'}]),
+        )
+        name_arm_1 = 'fadd'
+        operands_arm_1 = [
+            {'class': 'register', 'prefix': 'v', 'shape': 's'},
+            {'class': 'register', 'prefix': 'v', 'shape': 's'},
+            {'class': 'register', 'prefix': 'v', 'shape': 's'},
+        ]
+        instr_form_arm_1 = test_mm_arm.get_instruction_from_dict(name_arm_1, operands_arm_1)
+        self.assertEqual(instr_form_arm_1, test_mm_arm.get_instruction(name_arm_1, operands_arm_1))
+        self.assertEqual(
+            test_mm_arm.get_instruction_from_dict('b.ne', [{'class': 'identifier'}]),
+            test_mm_arm.get_instruction('b.ne', [{'class': 'identifier'}]),
+        )
+
+        # test full instruction name
+        self.assertEqual(
+            MachineModel.get_full_instruction_name(instr_form_x86_1),
+            'vaddpd  register(name:xmm),register(name:xmm),register(name:xmm)',
+        )
+        self.assertEqual(
+            MachineModel.get_full_instruction_name(instr_form_arm_1),
+            'fadd  register(prefix:v,shape:s),register(prefix:v,shape:s),'
+            + 'register(prefix:v,shape:s)',
+        )
+
+        # test get_store_tp
+        self.assertEqual(
+            test_mm_x86.get_store_throughput(
+                {'base': {'name': 'x'}, 'offset': None, 'index': None, 'scale': 1}
+            ),
+            [[2, '237'], [2, '4']],
+        )
+        self.assertEqual(
+            test_mm_x86.get_store_throughput(
+                {'base': {'prefix': 'NOT_IN_DB'}, 'offset': None, 'index': 'NOT_NONE', 'scale': 1}
+            ),
+            [[1, '23'], [1, '4']],
+        )
+        self.assertEqual(
+            test_mm_arm.get_store_throughput(
+                {'base': {'prefix': 'x'}, 'offset': None, 'index': None, 'scale': 1}
+            ),
+            [[2, '34'], [2, '5']],
+        )
+        self.assertEqual(
+            test_mm_arm.get_store_throughput(
+                {'base': {'prefix': 'NOT_IN_DB'}, 'offset': None, 'index': None, 'scale': 1}
+            ),
+            [[1, '34'], [1, '5']],
+        )
+
+        # test get_store_lt
+        self.assertEqual(
+            test_mm_x86.get_store_latency(
+                {'base': {'name': 'x'}, 'offset': None, 'index': None, 'scale': '1'}
+            ),
+            0,
+        )
+        self.assertEqual(
+            test_mm_arm.get_store_latency(
+                {'base': {'prefix': 'x'}, 'offset': None, 'index': None, 'scale': '1'}
+            ),
+            0,
+        )
+
+        # test has_hidden_load
+        self.assertFalse(test_mm_x86.has_hidden_loads())
+
+        # test default load tp
+        self.assertEqual(
+            test_mm_x86.get_load_throughput(
+                {'base': {'name': 'x'}, 'offset': None, 'index': None, 'scale': 1}
+            ),
+            [[1, '23'], [1, ['2D', '3D']]],
+        )
+
+        # test adding port
+        test_mm_x86.add_port('dummyPort')
+        test_mm_arm.add_port('dummyPort')
+
+        # test dump of DB
+        with open('/dev/null', 'w') as dev_null:
+            test_mm_x86.dump(stream=dev_null)
+            test_mm_arm.dump(stream=dev_null)
+
     def test_src_dst_assignment_x86(self):
         for instruction_form in self.kernel_x86:
             with self.subTest(instruction_form=instruction_form):
@@ -106,52 +232,79 @@ class TestSemanticTools(unittest.TestCase):
                 self.assertIsInstance(instruction_form['port_pressure'], list)
                 self.assertEqual(len(instruction_form['port_pressure']), port_num)
 
+    def test_optimal_throughput_assignment(self):
+        # x86
+        kernel_fixed = deepcopy(self.kernel_x86)
+        self.semantics_csx.add_semantics(kernel_fixed)
+        self.assertEqual(get_unmatched_instruction_ratio(kernel_fixed), 0)
+        kernel_optimal = deepcopy(kernel_fixed)
+        self.semantics_csx.assign_optimal_throughput(kernel_optimal)
+        tp_fixed = self.semantics_csx.get_throughput_sum(kernel_fixed)
+        tp_optimal = self.semantics_csx.get_throughput_sum(kernel_optimal)
+        self.assertNotEqual(tp_fixed, tp_optimal)
+        self.assertTrue(max(tp_optimal) <= max(tp_fixed))
+
+        # arm
+        kernel_fixed = deepcopy(self.kernel_AArch64)
+        self.semantics_tx2.add_semantics(kernel_fixed)
+        self.assertEqual(get_unmatched_instruction_ratio(kernel_fixed), 0)
+        kernel_optimal = deepcopy(kernel_fixed)
+        self.semantics_tx2.assign_optimal_throughput(kernel_optimal)
+        tp_fixed = self.semantics_tx2.get_throughput_sum(kernel_fixed)
+        tp_optimal = self.semantics_tx2.get_throughput_sum(kernel_optimal)
+        self.assertNotEqual(tp_fixed, tp_optimal)
+        self.assertTrue(max(tp_optimal) <= max(tp_fixed))
+
     def test_kernelDG_x86(self):
         #
-        #  3
-        #   \___>5__>6
+        #  4
+        #   \___>6__>7
         #   /
-        #  2
-        #     4_______>8
+        #  3
+        #     5_______>9
         #
         dg = KernelDG(self.kernel_x86, self.parser_x86, self.machine_model_csx)
         self.assertTrue(nx.algorithms.dag.is_directed_acyclic_graph(dg.dg))
+        self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=3))), 1)
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=3)), 6)
         self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=4))), 1)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=4)), 7)
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=4)), 6)
         self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=5))), 1)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=5)), 7)
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=5)), 9)
         self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=6))), 1)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=6)), 10)
-        self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=7))), 1)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=7)), 8)
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=6)), 7)
+        self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=7))), 0)
         self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=8))), 0)
-        self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=9))), 0)
         with self.assertRaises(ValueError):
             dg.get_dependent_instruction_forms()
+        # test dot creation
+        dg.export_graph(filepath='/dev/null')
 
     def test_kernelDG_AArch64(self):
         dg = KernelDG(self.kernel_AArch64, self.parser_AArch64, self.machine_model_tx2)
         self.assertTrue(nx.algorithms.dag.is_directed_acyclic_graph(dg.dg))
-        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=4)), {8, 9})
-        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=5)), {10, 11})
-        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=6)), {7, 8, 9})
-        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=7)), {10, 11})
+        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=3)), {7, 8})
+        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=4)), {9, 10})
+        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=5)), {6, 7, 8})
+        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=6)), {9, 10})
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=7)), 13)
         self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=8)), 14)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=9)), 15)
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=9)), 16)
         self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=10)), 17)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=11)), 18)
-        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=12)), {14, 15})
-        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=13)), {17, 18})
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=14)), 16)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=15)), 16)
-        self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=16))), 0)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=17)), 19)
-        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=18)), 19)
+        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=11)), {13, 14})
+        self.assertEqual(set(dg.get_dependent_instruction_forms(line_number=12)), {16, 17})
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=13)), 15)
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=14)), 15)
+        self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=15))), 0)
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=16)), 18)
+        self.assertEqual(next(dg.get_dependent_instruction_forms(line_number=17)), 18)
+        self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=18))), 0)
         self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=19))), 0)
         self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=20))), 0)
-        self.assertEqual(len(list(dg.get_dependent_instruction_forms(line_number=21))), 0)
         with self.assertRaises(ValueError):
             dg.get_dependent_instruction_forms()
+        # test dot creation
+        dg.export_graph(filepath='/dev/null')
 
     def test_hidden_load(self):
         machine_model_hld = MachineModel(
@@ -185,12 +338,12 @@ class TestSemanticTools(unittest.TestCase):
             dg.get_loopcarried_dependencies()
 
     def test_loop_carried_dependency_x86(self):
-        lcd_id = 9
-        lcd_id2 = 6
+        lcd_id = 8
+        lcd_id2 = 5
         dg = KernelDG(self.kernel_x86, self.parser_x86, self.machine_model_csx)
         lc_deps = dg.get_loopcarried_dependencies()
         self.assertEqual(len(lc_deps), 2)
-        # ID 9
+        # ID 8
         self.assertEqual(
             lc_deps[lcd_id]['root'], dg.dg.nodes(data=True)[lcd_id]['instruction_form']
         )
@@ -198,7 +351,9 @@ class TestSemanticTools(unittest.TestCase):
         self.assertEqual(
             lc_deps[lcd_id]['dependencies'][0], dg.dg.nodes(data=True)[lcd_id]['instruction_form']
         )
-        # ID 6
+        # w/  flag dependencies: ID 9 w/ len=2
+        # w/o flag dependencies: ID 5 w/ len=1
+        # TODO discuss
         self.assertEqual(
             lc_deps[lcd_id2]['root'], dg.dg.nodes(data=True)[lcd_id2]['instruction_form']
         )
